
// This code implements all functions of the R02 PCB configured for RS-232 communication with a computer.
// WPI MQP Green Energy Applications
// Written by: Kurt Snieckus <ksnieck@alum.wpi.edu> with help from Nathaniel VerLee and Anthony Ducimo
// Project Advisor: Prof. John McNeill, WPI Electrical and Computer Engineering Department


#include <stdio.h>
#include <file.h>
#include <msp430.h>
#include <msp430x26x.h>
#include "adc_spi.h"
#include "dac.h"
#include "uart.h"


void mppt(unsigned long pin);

// Un-calibrated ADC measurement which the converter should be off when it is below
#define VIN_MIN 1500

// Maximum output value of DAC, this value also causes the least power to be transfered through system
#define DAC_MAX 0xFFF

// POWER LIMIT: Uncalibrated max pin value, used by mppt(), for power limiting on small loads
//#define P_LIMIT 6000
char plimiting = 0;
int plimit = 82335;		// 100 watts

// Average over ADC readings
#define ADC_AVG 10

int main(void)
{

	WDTCTL = WDTPW + WDTHOLD;	// Stop watchdog timer

	// Setup clocks for
	BCSCTL1 = CALBC1_16MHZ;		// Set DCO
	DCOCTL = CALDCO_16MHZ;
	__delay_cycles(100000); 	// Wait for DCO to settle?!

	P4DIR |= 0xFF;				// LEDs
	P4OUT = 0x80;				// Turn on first LED
	P1DIR |= 0xFD;				// Port 1 is all outputs except 1.1
	P1REN = 0; 					// Don't need pull up/down
	P1OUT = 0;					// Set everything 0?

	adc_spi_init();  			// Configure spi interface to ADC
	pc_uart_init();				// Initialize UART interface to PC
	dac_init();					// Initialize DAC interface to converter

	crazy_leds();				// Do something cray!

	add_device("uart", _SSA, uart_open, uart_close, uart_read, uart_write, uart_lseek, uart_unlink, uart_rename);

	FILE * fid = fopen("uart","w");
	fprintf(fid,"Hello, world\n");

	/*FILE * fid = fopen("uart","w");
	freopen("uart:", "w", stdout); // redirect stdout to uart
	setvbuf(stdout, NULL, _IONBF, 0); // turn off buffering for stdout
	printf("Hello world!\r\n");*/

	unsigned long delayl = 160;
	unsigned long i, loop;
	unsigned long pinmax,poutmax;
	pinmax = 0;
	poutmax = 0;

	// Loop Forever!
	while (1) {

		unsigned long vin,vout,iin,iout,pin,pout;
		unsigned long vin_val, vout_val, iin_val, iout_val, pin_val, pout_val;

		if (P1IN & 0x02) P4OUT |= 0x40;
		else P4OUT &= ~(0x40);

		// Measure ADC V and I, In and Out
		// Average over ADC_AVG readings

		// Channel 0
		adc_reading(0, 0, 2);	// Start config load and reading taking with out storing garbage
		iout = adc_reading(0, 1, ADC_AVG);
		vout = adc_reading(1, 2, ADC_AVG);
		iin = adc_reading(2, 3, ADC_AVG);
		vin = adc_reading(3, 0, ADC_AVG);

		// Calculate power for stuff, chops bottom 8 bits off v and i and produces a 16 bit power
		pin = (vin >> 8) * (iin >> 8);
		if (pin > pinmax) pinmax = pin;
		//pout = (vout >> 8) * (iout >> 8);
		//if (pout > poutmax) poutmax = pout;

		// Do stuff based on that
		//if (vin < VIN_MIN || ~(P1IN) & 0x02) {
		if (vin < VIN_MIN) {
			// shutoff converter
			P1OUT &= ~(0x01);	// Set
			P4OUT &= ~(0x20);   // Turn off converter enable LED
			// Set DAC to Max (smallest power of converter)
			dac_write(0, DAC_MAX);
		} else {
			//enable converter
			P1OUT |= 0x01;
			P4OUT |= 0x20;   // Turn on converter enable LED
			// Modify DAC to track MPPT
			mppt(pin);
		}

		// Print output
		if (loop > 100) {	// Only print once every 100 samples, to keep the computer sane
			pc_uart_tx(0xFF);	// Start Word
			pc_uart_tx(0xFF);

			// vin
			pc_uart_tx((char) (0xFF & vin));  // Transmit LSByte
			pc_uart_tx((char) (0xFF & (vin >> 8))); // Transmit MSByte


			// iin
			pc_uart_tx((char) (0xFF & iin));  // Transmit LSByte
			pc_uart_tx((char) (0xFF & (iin >> 8))); // Transmit MSByte


			// vout
			pc_uart_tx((char) (0xFF & (vout >> 8))); // Transmit MSByte
			pc_uart_tx((char) (0xFF & vout));  // Transmit LSByte

			// iout
			pc_uart_tx((char) (0xFF & (iout >> 8))); // Transmit MSByte
			pc_uart_tx((char) (0xFF & iout));  // Transmit LSByte


			// DAC output
			pc_uart_tx((char) (0xFF & DAC12_0DAT));
			pc_uart_tx((char) (0xFF & (DAC12_0DAT >> 8)));
			loop = 0;
		}
		loop++;

		// Delay to control MPPT update rate
		for (i = 0; i < delayl; i++);

	}
}

// Maximum Power Point Tracking Algorithm
// takes the current converter input power measured by the ADC and adjusts the DAC to find
// the point of maximum power.
void mppt(unsigned long pin) {
	static unsigned long pprev;
	static unsigned char dacdir; // Direction of last dac movement, 1 is inc, 0 is dec

	// Limit, if the power is over the limit, then increase dac (decrease power)
	if (pin > plimit) {
		if (DAC12_0DAT < 0xFFF) {
			DAC12_0DAT++;
		}
		plimiting = 1;
		return;
	}

	// MPPT algorithm, if pin has increased, move dac in same direction,
	// if pin has decreased since last run, move dac in opposite direction

	if(pin < pprev) {
		// if power has decreased we're going to switch the direction of dacdir
		dacdir = 1-dacdir;
	}

	if (pin < 1000) dacdir = 0; //

	pprev = pin; // fill pprev with pin because we've used it this iteration

	// DAC CHANNEL 0
	if (dacdir == 1) {// we moved the dac up last time and we want to again
		if (DAC12_0DAT < 0xFFF) {
			DAC12_0DAT++;
		}
		if (DAC12_0DAT < 0xFFF) {
			DAC12_0DAT++;
		}
	}
	else { // we moved the dac down last time and we want to again
		if (DAC12_0DAT > 0) {
			DAC12_0DAT--;
		}
		if (DAC12_0DAT > 0) {
			DAC12_0DAT--;
		}
	}

}
